@@ -181,6 +181,11 @@ lList' [] = []
 lList' [p] = [if isNullSpan p then (p,"") else (p,"}")]
 lList' (p:ps) = (if isNullSpan p then (p,"") else (p,";")) : lList' ps
 
+printSemi :: SrcSpan -> EP ()
+printSemi p = do
+  printWhitespace (pos p)
+  when (not $ isNullSpan p) $ printString ";"
+                
 
 --------------------------------------------------
 -- Exact printing
@@ -1117,16 +1122,36 @@ instance ExactP Exp where
             printStringAt (pos b) "in"
             exactPC e
          _ -> errorEP "ExactP: Exp: Let is given wrong number of srcInfoPoints"
-    If l ec et ee   ->
+    If l ec et ee   -> -- traceShow (srcInfoPoints l) $ do
+        -- First we need to sort out if there are any optional
+        -- semicolons hiding among the srcInfoPoints.
         case srcInfoPoints l of
-         [a,b,c] -> do
-            printString "if"
-            exactPC ec
-            printStringAt (pos b) "then"
-            exactPC et
-            printStringAt (pos c) "else"
-            exactPC ee
-         _ -> errorEP "ExactP: Exp: If is given wrong number of srcInfoPoints"
+         (pIf:b:c:rest) -> do
+            let (mpSemi1,pThen,rest2) =
+                           if snd (spanSize b) == 4 -- this is "then", not a semi
+                            then (Nothing, b, c:rest)
+                            else (Just b, c, rest)
+            case rest2 of
+              (c:rest3) -> do
+                let (mpSemi2,rest4) = if snd (spanSize c) == 4 -- this is "else", not a semi
+                                       then (Nothing, rest2) 
+                                       else (Just c, rest3)
+                case rest4 of
+                  [pElse] -> do
+                    -- real work starts here:
+                    printString "if"
+                    exactPC ec
+                    maybeEP printSemi  mpSemi1
+                    printStringAt (pos pThen) "then"
+                    exactPC et
+                    maybeEP printSemi mpSemi2
+                    printStringAt (pos pElse) "else"
+                    exactPC ee
+                  [] -> errorEP "ExactP: Exp: If is given too few srcInfoPoints"
+                  _  -> errorEP "ExactP: Exp: If is given too many srcInfoPoints"
+              _ -> errorEP "ExactP: Exp: If is given too few srcInfoPoints"
+
+         _ -> errorEP "ExactP: Exp: If is given too few srcInfoPoints"
     Case l e alts   ->
         case srcInfoPoints l of
          a:b:pts -> do
@@ -1251,10 +1276,10 @@ instance ExactP Exp where
     SpliceExp l sp  -> exactP sp
     QuasiQuote l name qt    -> do
         let qtLines = lines qt
-        printString $ "[$" ++ name ++ "|"
+        printString $ "[" ++ name ++ "|"
         sequence_ (intersperse newLine $ map printString qtLines)
         printString "|]"
-    XTag l xn attrs mat es  ->
+    XTag l xn attrs mat es  -> do
         case srcInfoPoints l of
          [a,b,c,d,e] -> do
             printString "<"
@@ -1267,6 +1292,20 @@ instance ExactP Exp where
             printWhitespace (pos d)
             exactP xn
             printStringAt (pos e) ">"
+         -- TODO: Fugly hack/duplication, should be refactored
+         -- For the case when there's an optional semicolon
+         [a,b,semi,c,d,e] -> do
+            printString "<"
+            exactPC xn
+            mapM_ exactPC attrs
+            maybeEP exactPC mat
+            printStringAt (pos b) ">"
+            mapM_ exactPC es
+            printSemi semi
+            printStringAt (pos c) "</"
+            printWhitespace (pos d)
+            exactP xn
+            printStringAt (pos e) ">"
          _ -> errorEP "ExactP: Exp: XTag is given wrong number of srcInfoPoints"
     XETag l xn attrs mat    ->
         case srcInfoPoints l of
@@ -1294,7 +1333,14 @@ instance ExactP Exp where
             mapM_ exactPC es
             printStringAt (pos b) "</"
             printStringAt (pos c) "%>"
-
+         -- Ugly duplication for when there's an optional semi
+         [a,semi,b,c] -> do
+            printString "<%>"
+            mapM_ exactPC es
+            printSemi semi
+            printStringAt (pos b) "</"
+            printStringAt (pos c) "%>"
+         _ -> errorEP "ExactP: Exp: XChildTag is given wrong number of srcInfoPoints"
     CorePragma l      str e         ->
         case srcInfoPoints l of
          [a,b] -> do
@@ -1554,6 +1600,19 @@ instance ExactP Pat where
             printWhitespace (pos d)
             exactP xn
             printStringAt (pos e) ">"
+         -- Optional semi
+         [a,b,semi,c,d,e] -> do
+            printString "<"
+            exactPC xn
+            mapM_ exactPC attrs
+            maybeEP exactPC mat
+            printStringAt (pos b) ">"
+            mapM_ exactPC ps
+            printSemi semi
+            printStringAt (pos c) "</"
+            printWhitespace (pos d)
+            exactP xn
+            printStringAt (pos e) ">"
          _ -> errorEP "ExactP: Pat: PXTag is given wrong number of srcInfoPoints"
     PXETag l xn attrs mat ->
         case srcInfoPoints l of
@@ -1666,8 +1725,12 @@ instance ExactP Binds where
   exactP (IPBinds l ips) = layoutList (srcInfoPoints l) ips
 
 instance ExactP CallConv where
-  exactP (StdCall _) = printString "stdcall"
-  exactP (CCall   _) = printString "ccall"
+  exactP (StdCall   _) = printString "stdcall"
+  exactP (CCall     _) = printString "ccall"
+  exactP (CPlusPlus _) = printString "cplusplus"
+  exactP (DotNet    _) = printString "dotnet"
+  exactP (Jvm       _) = printString "jvm"
+  exactP (Js        _) = printString "js"
 
 instance ExactP Safety where
   exactP (PlayRisky _) = printString "unsafe"
