@@ -404,8 +404,12 @@ sSafety (PlayRisky _) = S.PlayRisky
 sSafety (PlaySafe _ b) = S.PlaySafe b
 
 sCallConv :: CallConv l -> S.CallConv
-sCallConv (StdCall _) = S.StdCall
-sCallConv (CCall _)   = S.CCall
+sCallConv (StdCall   _) = S.StdCall
+sCallConv (CCall     _) = S.CCall
+sCallConv (CPlusPlus _) = S.CPlusPlus
+sCallConv (DotNet    _) = S.DotNet
+sCallConv (Jvm       _) = S.Jvm
+sCallConv (Js        _ ) = S.Js
 
 sModulePragma :: SrcInfo loc => ModulePragma loc -> S.ModulePragma
 sModulePragma pr = case pr of
