@@ -234,10 +234,16 @@ checkClassHeader t = do
     return (Nothing,dh)
 
 checkSimple :: String -> PType L -> [TyVarBind L] -> P (DeclHead L)
-checkSimple kw (TyApp _ l t) xs | isTyVarBind t = checkSimple kw l (toTyVarBind t : xs)
-checkSimple _  (TyInfix l t1 (UnQual _ t) t2) []
-    | isTyVarBind t1 && isTyVarBind t2 =
-       checkEnabled TypeOperators >> return (DHInfix l (toTyVarBind t1) t (toTyVarBind t2))
+--checkSimple kw (TyApp _ l t) xs | isTyVarBind t = checkSimple kw l (toTyVarBind t : xs)
+
+checkSimple kw x@(TyApp _ l t) xs = do
+  tvb <- mkTyVarBind kw t
+  checkSimple kw l (tvb : xs)
+checkSimple kw  x@(TyInfix l t1 (UnQual _ t) t2) [] = do
+       checkEnabled TypeOperators
+       tv1 <- mkTyVarBind kw t1
+       tv2 <- mkTyVarBind kw t2
+       return (DHInfix l tv1 t tv2)
 checkSimple _kw (TyCon l (UnQual _ t))   xs = do
     case t of
       Symbol _ _ -> checkEnabled TypeOperators
@@ -246,16 +252,26 @@ checkSimple _kw (TyCon l (UnQual _ t))   xs = do
 checkSimple kw (TyParen l t) xs = do
     dh <- checkSimple kw t xs
     return (DHParen l dh)
-checkSimple kw _ _ = fail ("Illegal " ++ kw ++ " declaration")
+checkSimple kw _l _ = fail ("Illegal " ++ kw ++ " declaration")
 
+mkTyVarBind :: String -> PType L -> P (TyVarBind L)
+mkTyVarBind _ (TyVar l n) = return $ UnkindedVar l n
+mkTyVarBind _ (TyKind l (TyVar _ n) k) = return $ KindedVar l n k
+mkTyVarBind _ (TyCon l (UnQual _ n@(Symbol _ _))) = checkEnabled TypeOperators >> return (UnkindedVar l n)
+mkTyVarBind _ (TyKind l (TyCon _ (UnQual _ n@(Symbol _ _))) k) = checkEnabled TypeOperators >> return (KindedVar l n k)
+mkTyVarBind kw _ = fail ("Illegal " ++ kw ++ " declaration")
+
+{-
 isTyVarBind :: PType L -> Bool
 isTyVarBind (TyVar _ _) = True
+--isTyVarBind (TyCon _ (UnQual _ n@(Symbol _ _))) = True
 isTyVarBind (TyKind _ (TyVar _ _) _) = True
 isTyVarBind _ = False
 
 toTyVarBind :: PType L -> TyVarBind L
 toTyVarBind (TyVar l n) = UnkindedVar l n
 toTyVarBind (TyKind l (TyVar _ n) k) = KindedVar l n k
+-}
 
 checkInstHeader :: PType L -> P (Maybe (S.Context L), InstHead L)
 checkInstHeader (TyForall _ Nothing cs t) = do
@@ -314,6 +330,7 @@ checkPat e [] = case e of
                     r <- checkPat r []
                     return (PInfixApp loc l c r)
             QVarOp ppos (UnQual _ (Symbol _ "+")) -> do
+                    checkEnabled NPlusKPatterns
                     case (l,r) of
                         (Var _ (UnQual _ n@(Ident _ _)), Lit _ (Int kpos k _)) -> do
                             let pp = srcInfoSpan ppos
@@ -732,7 +749,7 @@ checkInstBody decls = do
 checkMethodDef :: Decl L -> P ()
 checkMethodDef (PatBind _ (PVar _ _) _ _ _) = return ()
 checkMethodDef (PatBind loc _ _ _ _) =
-    fail "illegal method definition" -- `atSrcLoc` loc
+    fail "illegal method definition" `atSrcLoc` fromSrcInfo loc
 checkMethodDef _ = return ()
 
 -----------------------------------------------------------------------------
@@ -786,7 +803,7 @@ checkRevDecls = mergeFunBinds []
             | name' =~= name =
             if length ps' /= arity
             then fail ("arity mismatch for '" ++ prettyPrint name ++ "'")
-                    -- `atSrcLoc` loc
+                    `atSrcLoc` fromSrcInfo loc
             else mergeMatches (ms++ms') ds (loc <++> l)
         mergeMatches ms' ds l = mergeFunBinds (FunBind l ms':revDs) ds
     mergeFunBinds revDs (FunBind l ims1@(InfixMatch _ _ name _ _ _:_):ds1) =
@@ -810,7 +827,7 @@ checkRevClsDecls = mergeClsFunBinds []
             | name' =~= name =
             if length ps' /= arity
             then fail ("arity mismatch for '" ++ prettyPrint name ++ "'")
-                    -- `atSrcLoc` loc
+                    `atSrcLoc` fromSrcInfo loc
             else mergeMatches (ms++ms') ds (loc <++> l)
         mergeMatches ms' ds l = mergeClsFunBinds (ClsDecl l (FunBind l ms'):revDs) ds
     mergeClsFunBinds revDs (ClsDecl l (FunBind _ ims1@(InfixMatch _ _ name _ _ _:_)):ds1) =
@@ -835,7 +852,7 @@ checkRevInstDecls = mergeInstFunBinds []
             | name' =~= name =
             if length ps' /= arity
             then fail ("arity mismatch for '" ++ prettyPrint name ++ "'")
-                    -- `atSrcLoc` loc
+                    `atSrcLoc` fromSrcInfo loc
             else mergeMatches (ms++ms') ds (loc <++> l)
         mergeMatches ms' ds l = mergeInstFunBinds (InsDecl l (FunBind l ms'):revDs) ds
     mergeInstFunBinds revDs (InsDecl l (FunBind _ ims1@(InfixMatch _ _ name _ _ _:_)):ds1) =
@@ -876,11 +893,11 @@ checkType t = checkT t False
 checkT :: PType L -> Bool -> P (S.Type L)
 checkT t simple = case t of
     TyForall l tvs@Nothing cs pt    -> do
-            when (simple) $ checkEnabled ExplicitForall
+            when (simple) $ checkEnabled ExplicitForAll
             ctxt <- checkContext cs
             check1Type pt (S.TyForall l Nothing ctxt)
     TyForall l tvs cs pt -> do
-            checkEnabled ExplicitForall
+            checkEnabled ExplicitForAll
             ctxt <- checkContext cs
             check1Type pt (S.TyForall l tvs ctxt)
     TyFun   l at rt   -> check2Types at rt (S.TyFun l)
