@@ -589,8 +589,12 @@ instance Pretty Safety where
         pretty (PlaySafe b)     = text $ if b then "threadsafe" else "safe"
 
 instance Pretty CallConv where
-        pretty StdCall  = text "stdcall"
-        pretty CCall    = text "ccall"
+        pretty StdCall   = text "stdcall"
+        pretty CCall     = text "ccall"
+        pretty CPlusPlus = text "cplusplus"
+        pretty DotNet    = text "dotnet"
+        pretty Jvm       = text "jvm"
+        pretty Js        = text "js"
 
 ------------------------- Pragmas ---------------------------------------
 ppWarnDepr :: ([Name], String) -> Doc
@@ -767,7 +771,7 @@ instance Pretty Exp where
         prettyPrec p (NegApp e) = parensIf (p > 0) $ char '-' <> prettyPrec 4 e
         prettyPrec p (App a b) = parensIf (p > 3) $ myFsep [prettyPrec 3 a, prettyPrec 4 b]
         prettyPrec p (Lambda _loc patList ppBody) = parensIf (p > 1) $ myFsep $
-                char '\\' : map pretty patList ++ [text "->", pretty ppBody]
+                char '\\' : map (prettyPrec 2) patList ++ [text "->", pretty ppBody]
         -- keywords
         -- two cases for lets
         prettyPrec p (Let (BDecls declList) letBody) =
@@ -825,7 +829,7 @@ instance Pretty Exp where
         prettyPrec _ (SpliceExp s) = pretty s
         prettyPrec _ (TypQuote t)  = text "\'\'" <> pretty t
         prettyPrec _ (VarQuote x)  = text "\'" <> pretty x
-        prettyPrec _ (QuasiQuote n qt) = text ("[$" ++ n ++ "|" ++ qt ++ "|]")
+        prettyPrec _ (QuasiQuote n qt) = text ("[" ++ n ++ "|" ++ qt ++ "|]")
         -- Hsx
         prettyPrec _ (XTag _ n attrs mattr cs) =
                 let ax = maybe [] (return . pretty) mattr
@@ -1095,6 +1099,7 @@ specialName FunCon = "->"
 specialName (TupleCon b n) = "(" ++ hash ++ replicate (n-1) ',' ++ hash ++ ")"
     where hash = if b == Unboxed then "#" else ""
 specialName Cons = ":"
+specialName UnboxedSingleCon = "(# #)"
 
 ppContext :: Context -> Doc
 ppContext []      = empty
