@@ -86,7 +86,9 @@ module Language.Haskell.Exts.Syntax (
     unit_con, tuple_con, unboxed_singleton_con,
     -- ** Special identifiers
     as_name, qualified_name, hiding_name, minus_name, bang_name, dot_name, star_name,
-    export_name, safe_name, unsafe_name, threadsafe_name, stdcall_name, ccall_name,
+    export_name, safe_name, unsafe_name, threadsafe_name, 
+    stdcall_name, ccall_name, cplusplus_name, dotnet_name, jvm_name, js_name,
+    forall_name, family_name,
     -- ** Type constructors
     unit_tycon_name, fun_tycon_name, list_tycon_name, tuple_tycon_name, unboxed_singleton_tycon_name,
     unit_tycon, fun_tycon, list_tycon, tuple_tycon, unboxed_singleton_tycon,
@@ -714,6 +716,10 @@ data Safety
 data CallConv
     = StdCall
     | CCall
+    | CPlusPlus
+    | DotNet
+    | Jvm
+    | Js
 #ifdef __GLASGOW_HASKELL__
   deriving (Eq,Ord,Show,Typeable,Data)
 #else
@@ -973,13 +979,21 @@ bang_name      = Symbol "!"
 dot_name       = Symbol "."
 star_name      = Symbol "*"
 
-export_name, safe_name, unsafe_name, threadsafe_name, stdcall_name, ccall_name :: Name
+export_name, safe_name, unsafe_name, threadsafe_name, 
+  stdcall_name, ccall_name, cplusplus_name, dotnet_name,
+  jvm_name, js_name, forall_name, family_name :: Name
 export_name     = Ident "export"
 safe_name       = Ident "safe"
 unsafe_name     = Ident "unsafe"
 threadsafe_name = Ident "threadsafe"
 stdcall_name    = Ident "stdcall"
 ccall_name      = Ident "ccall"
+cplusplus_name  = Ident "cplusplus"
+dotnet_name     = Ident "dotnet"
+jvm_name        = Ident "jvm"
+js_name         = Ident "js"
+forall_name     = Ident "forall"
+family_name     = Ident "family"
 
 unit_tycon_name, fun_tycon_name, list_tycon_name, unboxed_singleton_tycon_name :: QName
 unit_tycon_name = unit_con_name
